import pandas as pd
import re
import string


def create_corpus(df, id_col, text_cols=[]):
    """
    Ze vstupních dat vytvoří corpus (sloučené texty, pro textovou analýzu).

    Např. v rámci projektů corpus tvoří Název projektu, Cíle projektu případně i Klíčová slova.

    Parameters
    ----------
    df (DataFrame): vstupní DataFrame se surovými daty, ze kterých má vziknout corpus
    id_col (str): název sloupce, který slouží jako id (např. Kód projektu, ID dotazu)
    text_cols (list): název textových sloupců, ze kterých má vziknout corpus

    Returns
    -------
    df_corpus (DataFrame):

    """

    cols = [id_col] + text_cols
    df = df[cols]
    df = df.fillna('')  # agg a join nefunguje pokud jsou NA hodnoty
    df['corpus'] = df[df.columns[1:]].agg('; '.join, axis=1)
    df_corpus = df[[df.columns[0], df.columns[-1]]]
    df_corpus.columns = ['id', 'corpus']

    return df_corpus


def text_lemma(text, model):
    """
    Lemmatizuje text na základní tvar. Např. lepší -> dobrý, projektový -> projekt.

    Je potřeba mít stažený a načtený model - nejlépe czech-pdt-ud-2.5-191206.udpipe, dostupný na:
    https://lindat.mff.cuni.cz/repository/xmlui/handle/11234/1-3131

    Parameters
    ----------
    text (str): corpus (text) jednoho projektu, dotazu atd. => 1 buňka
    model (model): je potřeba mít nahraný předtrénovaný model (např. czech-pdt-ud-2.5-191206.udpipe)

    Returns
    -------
    lemma (str): lemmatizovaný text => 1 buňka

    """
    sentence = []

    for s in model.process(text):  # rozdělí text na věty
        for w in s.words:  # rozdělí text na slova
            if '<root>' not in w.lemma:  # <roo> je v podstatě začátek věty
                sentence.append(w.lemma)

    # složí zpátky celou větu z lemmatizovaných slov, resp. celý text.
    lemma = ' '.join(sentence)

    return lemma


def df_lemma(df_corpus, entity_name, model, df_lemma=None):
    """
    Vytvoří lemmatizovaný DataFrame a aktualizuje (nebo vytvoří) soubor (pickle) s lemmatizovanými texty.

    Nejdříve zkontroluje, jestli existují texty, které nebyly lemmatizované.
    Pro texty, které nebyly lemmatizované provede lemmatizaci.
    Aktualizuje původní soubor s lemmatizovanými texty o nově lemmatizované texty.

    !!!POZOR!!! Pro velké množství texu může zabrat nějakou dobu (cca 0.5 sec na text => 10 000 textů za 48 minut)

    Parameters
    ----------
    df_corpus (DataFrame): dataframe se všemi texty (corpus)
    entity_name (str): entita, které se lemmatizace týká (např. projekty), slouží pro pojmenování souboru
    model (model): je potřeba mít nahraný předtrénovaný model (např. czech-pdt-ud-2.5-191206.udpipe)
    df_lemma (DataFrame): nepovinný parametr, dataframe s lemmatizovanými texty (původní soubor pickle)

    Returns
    -------
    df_lemma (DataFrame): aktualizovaný dataframe lemmatizovaných textů

    """

    # pokud df_lemma neexistuje, pak lemmatizuje všechny texty a vytvoří pickle soubor
    if df_lemma is None:
        df_corpus['lemma'] = df_corpus['corpus'].apply(lambda x: text_lemma(x, model))
        df_lemma = df_corpus[[df_corpus.columns[0], 'lemma']]
        df_lemma.to_pickle(entity_name + '_lemma.pkl')

    # pokud existuje, tak lemmatizuje pouze nové texty a 'aktualizuje' pickle soubor
    else:
        # zjištění, které texty jsou nové
        id_set = set(df_corpus[df_corpus.columns[0]]) - set(df_lemma[df_lemma.columns[0]])

        # vyfiltruje pouze nové texty a provede lemmatizaci funkce cz_lemma
        df = df_corpus[df_corpus[df_corpus.columns[0]].isin(id_set)]
        df['lemma'] = df['corpus'].apply(lambda x: text_lemma(x, model))

        # rozšíří df původních lemmatizovaných textů a vytvoří nový soubor se stejným jménem
        df = df[[df.columns[0], 'lemma']]
        df_lemma = pd.concat([df_lemma, df], ignore_index=True)
        df_lemma.to_pickle(entity_name + '_lemma.pkl')

    return df_lemma

def text_cleaning(text):
    """ Preprocessing textu, odstranění netextových znaků."""

    # TODO: Zamyslet se, jestli se nedá čistit komplexněji v rámci jednoho regex

    text = text.lower()  # převední na malá písmena
    text = re.sub(';', ' ; ', text)
    text = re.sub(',', ' , ', text)
    text = re.sub('-', ' - ', text)
    text = re.sub('\d', '', text)  # odstranění čísel
    text = re.sub('[%s]' % re.escape(string.punctuation), ' ', text)

    return text

def df_cleaning(df_lemma):
    """
    Na vybraných projektech, dle filtru provede text preprocessing

    Parameters
    ----------
    df_lemma (DataFrame): lemmatizované texty projektů

    Returns
    -------
    df_cleaned (DataFrame): DataFrame připravený na další kroky analýzy podobností

    """
    # TODO: zapojit již do funkce lemmatizace a provádět text preprocessing jako celek
    # zde nechat jen výběr podle filtru VS

    #     df = df_lemma[(df_lemma['kod'].str.slice(0,4).isin(filtr_vs))]
    df_lemma['lemma'] = pd.DataFrame(df_lemma['lemma'].apply(lambda x: text_cleaning(x)))

    df_cleaned = df_lemma[['id', 'lemma']].reset_index(drop=True)

    return df_cleaned


import pandas as pd
import re
import os


def multiload(path, filename):
    """
    Načte a spojí všechny soubory se stejným jménem a strukturou v dané složce.

    Vhodné využít pro soubory, které jsou strukturou stejné a název se liší jen příponou.
    (Např. Export_projekty.xlsx, Export_projekty_MV.xlsx, Export_projekty_MPO.xlsx).

    Obsahuje i ztlumení varování "Workbook contains no default style, apply openpyxl's default".
    Pokud se podaří odstranit na straně pandas, tak odstranit i zde.

    Parameters
    ----------
    path (str): cesta, kde se dané soubory nachází
    filename (str): textový řetezec, který se nachází v názvu souborů

    Returns
    -------
    df_concat (DataFrame): DataFrame spojený z jednotlivých souborů

    """
    import warnings

    with warnings.catch_warnings(record=True):
        warnings.simplefilter('always')

        # vytvoří seznam všech souboru s daným názvem
        files = [file for file in os.listdir(path) if re.match(filename, file)]

        df_concat = pd.DataFrame()

        # postupně načte a spojí všechny soubory ze seznamu výše
        for file in files:
            df = pd.read_excel(os.path.join(path, file))
            df_concat = pd.concat([df_concat, df])

        return df_concat


import pandas as pd
import json


def ciselnikJSON(file_name):
    """
    Na základě vstupních dat vygeneruje JSON strukturu číselníku pro Otevřená data (OD).

    Z názvu nahraného souboru vytvoří název pro výstupní soubor (ve formátu - malá písmena, s diakritikou, spojená pomlčkou).
    Propisuje se také do iri datové sady i jendotlivých položek. Dále vytvoří prázdný dict s přednastavenou hlavičkou a částí
    "položky", která se iterativně naplní daty ze zdrojového souboru.


    Parameters
    ----------
    file_name (str): název souboru (musí odpovídat názvu datové sady v OD), kde jsou vstupní data.
                     Struktura - Kód, Název, Popis.

    Returns
    -------
    JSON file (file): výstupní soubor ve formátu JSON
    ciselnik_dict (dict): dict se strukturou číselníku, slouží hlavně k zobrazení výsledku

    """

    df = pd.read_excel(file_name)

    name = file_name.split('.')[0]  # název bez přípony (např. xlsx)

    json_file_name = '-'.join(name.lower().split(' ')) + '.json'  # název spojený pomlčkami, malá písmena

    iri = 'https://www.tacr.cz/opendata/číselníky/' + json_file_name

    # základní struktura dict
    # předvyplněná hlavička
    ciselnik_dict = {
        "@context": "https://ofn.gov.cz/číselníky/2022-02-08/kontexty/číselník.jsonld",
        "typ": "Číselník",
        "iri": iri,
        "název": {
            "cs": name
        },
        "položky": []
    }

    # naplnění části dict "položky"
    for i in range(len(df)):
        # data k naplnění
        kod = df.iloc[i, 0]
        nazev = df.iloc[i, 1]
        popis = df.iloc[i, 2]

        # naplnění dat
        child_dict = {
            "typ": "Položka",
            "iri": iri + '/' + str(kod),
            "kód": str(kod),
            "název": {
                "cs": nazev
            },
            "popis": {
                "cs": popis
            }
        }

        ciselnik_dict['položky'].append(child_dict)

    # vytvoření JSON souboru
    with open(json_file_name, 'w', encoding='utf-8') as f:
        json.dump(ciselnik_dict, f, ensure_ascii=False)

    return ciselnik_dict


def ciselnikCSV(file_name):
    """
    Na základě vstupních dat vygeneruje CSV strukturu číselníku pro Otevřená data (OD).

    Z názvu nahraného souboru vytvoří název pro výstupní soubor (ve formátu - malá písmena, s diakritikou, spojená pomlčkou).
    Propisuje se také do iri datové sady i jendotlivých položek. Dále vytvoří DataFrame s přednastavenou hlavičkou a částmi
    které se iterativně naplní daty ze zdrojového souboru.


    Parameters
    ----------
    file_name (str): název souboru (musí odpovídat názvu datové sady v OD), kde jsou vstupní data.
                     Struktura - Kód, Název, Popis.

    Returns
    -------
    CSV file (file): výstupní soubor ve formátu CSV
    ciselnik_df (DataFrame): DataFrame se strukturou číselníku, slouží hlavně k zobrazení výsledku

    """

    df = pd.read_excel(file_name)

    name = file_name.split('.')[0]  # název bez přípony (např. xlsx)

    csv_file_name = '-'.join(name.lower().split(' ')) + '.csv'  # název spojený pomlčkami, malá písmena

    iri = 'https://www.tacr.cz/opendata/číselníky/' + csv_file_name

    # standardizované názvy sloupců
    cols = ['číselník', 'číselník_název_cs', 'číselník_položka'
        , 'číselník_položka_kód', 'číselník_položka_název_cs', 'číselník_položka_popis_cs']

    ciselnik_list = []

    for i in range(len(df)):
        kod = int(df.iloc[i, 0])
        nazev = df.iloc[i, 1]
        popis = df.iloc[i, 2]

        iri_polozka = iri + '/' + str(kod)

        ciselnik_list.append([iri, name, iri_polozka, kod, nazev, popis])

    ciselnik_df = pd.DataFrame(ciselnik_list, columns=cols)
    ciselnik_df.to_csv(csv_file_name, encoding='utf-8', index=False)

    return ciselnik_df